package tk.eclipse.plugin.visualjsf.editparts;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.Action;

import tk.eclipse.plugin.visualjsf.VisualJSFPlugin;
import tk.eclipse.plugin.visualjsf.VisualJSFUtil;
import tk.eclipse.plugin.visualjsf.models.AbstractJSFModel;
import tk.eclipse.plugin.visualjsf.models.IValueChangeListenerModel;

public class CreateValueChangeListenerAction extends Action {
	
	private IValueChangeListenerModel model = null;
	private EditPartViewer viewer = null;
	
	public CreateValueChangeListenerAction(String id, String text, 
			IValueChangeListenerModel model, EditPartViewer viewer){
		super(text, VisualJSFPlugin.getImageDescriptor("icons/event_1.gif"));
		setId(id);
		this.model = model;
		this.viewer = viewer;
	}
	
	public void run(){
		IType beanType = ((AbstractJSFModel)this.model).getRoot().getBeanType();
		if(beanType==null){
			return;
		}
		final String beanName   = ((AbstractJSFModel)this.model).getRoot().getBeanName();
		final String methodName = getText();
		if(this.model.getValueChangeListener()==null || this.model.getValueChangeListener().length()==0){
			// Sets the method binding
			CommandStack stack = this.viewer.getEditDomain().getCommandStack();
			stack.execute(new Command(){
				public void execute(){
					CreateValueChangeListenerAction.this.model.setValueChangeListener("#{" + beanName + "." + methodName + "}");
				}
				public void undo() {
					CreateValueChangeListenerAction.this.model.setValueChangeListener("");
				}
			});
		}
		IMethod method = getMethod(beanType, methodName);
		if(method==null || !method.exists()){
			// Creates the action method
			VisualJSFUtil.createValueChangeMethod(beanType, methodName);
			method = getMethod(beanType, methodName);
		}
		VisualJSFUtil.openJavaElement(method);
	}
	
	private IMethod getMethod(IType type, String name){
		IMethod method = null;
		try {
			IMethod[] methods = type.getMethods();
			for(int i=0;i<methods.length;i++){
				if(methods[i].getElementName().equals(name)){
					method = methods[i];
					break;
				}
			}
		} catch(JavaModelException ex){
		}
		return method;
	}
}
